@@ -23,7 +23,14 @@ class CausalIdentifier:
     NONPARAMETRIC_NDE="nonparametric-nde"
     NONPARAMETRIC_NIE="nonparametric-nie"
     MAX_BACKDOOR_ITERATIONS = 100000
-    VALID_METHOD_NAMES = {"default", "exhaustive-search"}
+
+    # Backdoor method names
+    BACKDOOR_DEFAULT="default"
+    BACKDOOR_EXHAUSTIVE="exhaustive-search"
+    BACKDOOR_MIN="minimal-adjustment"
+    BACKDOOR_MAX="maximal-adjustment"
+    METHOD_NAMES = {BACKDOOR_DEFAULT, BACKDOOR_EXHAUSTIVE, BACKDOOR_MIN, BACKDOOR_MAX}
+    DEFAULT_BACKDOOR_METHOD = BACKDOOR_DEFAULT
 
     def __init__(self, graph, estimand_type,
             method_name = "default",
@@ -36,16 +43,23 @@ class CausalIdentifier:
         self._proceed_when_unidentifiable = proceed_when_unidentifiable
         self.logger = logging.getLogger(__name__)
 
-    def identify_effect(self):
+    def identify_effect(self, optimize_backdoor=False):
         """Main method that returns an identified estimand (if one exists).
 
         If estimand_type is non-parametric ATE, then  uses backdoor, instrumental variable and frontdoor identification methods,  to check if an identified estimand exists, based on the causal graph.
 
-        :param self: instance of the CausalEstimator class (or its subclass)
+        :param self: instance of the CausalIdentifier class (or its subclass)
         :returns:  target estimand, an instance of the IdentifiedEstimand class
         """
+        # First, check if there is a directed path from action to outcome
+        if not self._graph.has_directed_path(self.treatment_name, self.outcome_name):
+            self.logger.warn("No directed path from treatment to outcome. Causal Effect is zero.")
+            return IdentifiedEstimand(self,
+                    treatment_variable=self.treatment_name,
+                    outcome_variable=self.outcome_name,
+                    no_directed_path=True)
         if self.estimand_type == CausalIdentifier.NONPARAMETRIC_ATE:
-            return self.identify_ate_effect()
+            return self.identify_ate_effect(optimize_backdoor=optimize_backdoor)
         elif self.estimand_type == CausalIdentifier.NONPARAMETRIC_NDE:
             return self.identify_nde_effect()
         elif self.estimand_type == CausalIdentifier.NONPARAMETRIC_NIE:
@@ -56,13 +70,18 @@ class CausalIdentifier:
                 CausalIdentifier.NONPARAMETRIC_NDE,
                 CausalIdentifier.NONPARAMETRIC_NIE))
 
-    def identify_ate_effect(self):
+    def identify_ate_effect(self, optimize_backdoor):
         estimands_dict = {}
         mediation_first_stage_confounders = None
         mediation_second_stage_confounders = None
         ### 1. BACKDOOR IDENTIFICATION
         # First, checking if there are any valid backdoor adjustment sets
-        backdoor_sets = self.identify_backdoor(self.treatment_name, self.outcome_name)
+        if optimize_backdoor == False:
+            backdoor_sets = self.identify_backdoor(self.treatment_name, self.outcome_name)
+        else:
+            from dowhy.causal_identifiers.backdoor import Backdoor
+            path = Backdoor(self._graph._graph, self.treatment_name, self.outcome_name)
+            backdoor_sets = path.get_backdoor_vars()
         estimands_dict, backdoor_variables_dict = self.build_backdoor_estimands_dict(
                 self.treatment_name,
                 self.outcome_name,
@@ -70,9 +89,11 @@ class CausalIdentifier:
                 estimands_dict)
         # Setting default "backdoor" identification adjustment set
         default_backdoor_id = self.get_default_backdoor_set_id(backdoor_variables_dict)
-        estimands_dict["backdoor"] = estimands_dict.get(str(default_backdoor_id), None)
-        backdoor_variables_dict["backdoor"] = backdoor_variables_dict.get(str(default_backdoor_id), None)
-
+        if len(backdoor_variables_dict) > 0:
+            estimands_dict["backdoor"] = estimands_dict.get(str(default_backdoor_id), None)
+            backdoor_variables_dict["backdoor"] = backdoor_variables_dict.get(str(default_backdoor_id), None)
+        else:
+            estimands_dict["backdoor"] = None
         ### 2. INSTRUMENTAL VARIABLE IDENTIFICATION
         # Now checking if there is also a valid iv estimand
         instrument_names = self._graph.get_instruments(self.treatment_name,
@@ -230,68 +251,124 @@ class CausalIdentifier:
         )
         return estimand
 
-
-
-
-    def identify_backdoor(self, treatment_name, outcome_name):
+    def identify_backdoor(self, treatment_name, outcome_name,
+            include_unobserved=False, dseparation_algo="default"):
         backdoor_sets = []
-        backdoor_paths = self._graph.get_backdoor_paths(treatment_name, outcome_name)
+        backdoor_paths = None
+        bdoor_graph = None
+        if dseparation_algo == "naive":
+            backdoor_paths = self._graph.get_backdoor_paths(treatment_name, outcome_name)
+        elif dseparation_algo == "default":
+            bdoor_graph = self._graph.do_surgery(treatment_name,
+                    remove_outgoing_edges=True)
+        else:
+            raise ValueError(f"d-separation algorithm {dseparation_algo} is not supported")
+        method_name = self.method_name if self.method_name != CausalIdentifier.BACKDOOR_DEFAULT else CausalIdentifier.DEFAULT_BACKDOOR_METHOD
+
         # First, checking if empty set is a valid backdoor set
         empty_set = set()
-        check = self._graph.check_valid_backdoor_set(treatment_name, outcome_name, empty_set,
-                backdoor_paths=backdoor_paths)
+        check = self._graph.check_valid_backdoor_set(treatment_name,
+                outcome_name, empty_set,
+                backdoor_paths=backdoor_paths, new_graph=bdoor_graph,
+                dseparation_algo=dseparation_algo)
         if check["is_dseparated"]:
-            backdoor_sets.append({
-                'backdoor_set':empty_set,
-                'num_paths_blocked_by_observed_nodes': check["num_paths_blocked_by_observed_nodes"]})
-        # Second, checking for all other sets of variables
-        eligible_variables = self._graph.get_all_nodes() \
-            - set(treatment_name) \
-            - set(outcome_name) \
-            - set(self._graph.get_instruments(treatment_name, outcome_name))
-        eligible_variables -= self._graph.get_descendants(treatment_name)
+            backdoor_sets.append({'backdoor_set':empty_set})
+            # If the method is `minimal-adjustment`, return the empty set right away.
+            if method_name == CausalIdentifier.BACKDOOR_MIN:
+                return backdoor_sets
 
+        # Second, checking for all other sets of variables. If include_unobserved is false, then only observed variables are eligible.
+        eligible_variables = self._graph.get_all_nodes(include_unobserved=include_unobserved) \
+            - set(treatment_name) \
+            - set(outcome_name)
+        eligible_variables -= self._graph.get_descendants(treatment_name)
+        # If var is d-separated from both treatment or outcome, it cannot
+        # be a part of the backdoor set
+        filt_eligible_variables = set()
+        for var in eligible_variables:
+            dsep_treat_var = self._graph.check_dseparation(
+                    treatment_name, parse_state(var),
+                    set())
+            dsep_outcome_var = self._graph.check_dseparation(
+                    outcome_name, parse_state(var), set())
+            if not dsep_outcome_var or not dsep_treat_var:
+                filt_eligible_variables.add(var)
+        if method_name in CausalIdentifier.METHOD_NAMES:
+            backdoor_sets, found_valid_adjustment_set = self.find_valid_adjustment_sets(
+                    treatment_name, outcome_name,
+                    backdoor_paths, bdoor_graph,
+                    dseparation_algo,
+                    backdoor_sets, filt_eligible_variables,
+                    method_name=method_name,
+                    max_iterations= CausalIdentifier.MAX_BACKDOOR_ITERATIONS)
+            if method_name == CausalIdentifier.BACKDOOR_DEFAULT and found_valid_adjustment_set:
+                # repeat the above search with BACKDOOR_MIN
+                backdoor_sets, _ = self.find_valid_adjustment_sets(
+                        treatment_name, outcome_name,
+                        backdoor_paths, bdoor_graph,
+                        dseparation_algo,
+                        backdoor_sets, filt_eligible_variables,
+                        method_name=CausalIdentifier.BACKDOOR_MIN,
+                        max_iterations= CausalIdentifier.MAX_BACKDOOR_ITERATIONS)
+        else:
+            raise ValueError(f"Identifier method {method_name} not supported. Try one of the following: {CausalIdentifier.METHOD_NAMES}")
+        return backdoor_sets
+
+    def find_valid_adjustment_sets(self, treatment_name, outcome_name,
+            backdoor_paths, bdoor_graph, dseparation_algo,
+            backdoor_sets, filt_eligible_variables,
+            method_name, max_iterations):
         num_iterations = 0
         found_valid_adjustment_set = False
-        if self.method_name in CausalIdentifier.VALID_METHOD_NAMES:
-            for size_candidate_set in range(len(eligible_variables), 0, -1):
-                for candidate_set in itertools.combinations(eligible_variables, size_candidate_set):
-                    check = self._graph.check_valid_backdoor_set(treatment_name,
-                            outcome_name, candidate_set, backdoor_paths=backdoor_paths)
-                    self.logger.debug("Candidate backdoor set: {0}, is_dseparated: {1}, No. of paths blocked by observed_nodes: {2}".format(candidate_set, check["is_dseparated"], check["num_paths_blocked_by_observed_nodes"]))
-                    if check["is_dseparated"]:
-                        backdoor_sets.append({
-                            'backdoor_set': candidate_set,
-                            'num_paths_blocked_by_observed_nodes': check["num_paths_blocked_by_observed_nodes"]})
-                        if self._graph.all_observed(candidate_set):
-                            found_valid_adjustment_set = True
-                    num_iterations += 1
-                    if self.method_name == "default" and num_iterations > CausalIdentifier.MAX_BACKDOOR_ITERATIONS:
-                        break
-                if self.method_name == "default" and found_valid_adjustment_set:
+        all_nodes_observed = self._graph.all_observed(self._graph.get_all_nodes())
+        # If `minimal-adjustment` method is specified, start the search from the set with minimum size. Otherwise, start from the largest.
+        set_sizes = range(1, len(filt_eligible_variables) + 1, 1) if method_name == CausalIdentifier.BACKDOOR_MIN else range(len(filt_eligible_variables), 0, -1)
+        for size_candidate_set in set_sizes:
+            for candidate_set in itertools.combinations(filt_eligible_variables, size_candidate_set):
+                check = self._graph.check_valid_backdoor_set(treatment_name,
+                        outcome_name, candidate_set,
+                        backdoor_paths=backdoor_paths,
+                        new_graph = bdoor_graph,
+                        dseparation_algo = dseparation_algo)
+                self.logger.debug("Candidate backdoor set: {0}, is_dseparated: {1}".format(candidate_set, check["is_dseparated"]))
+                if check["is_dseparated"]:
+                    backdoor_sets.append({'backdoor_set': candidate_set})
+                    found_valid_adjustment_set = True
+                num_iterations += 1
+                if method_name == CausalIdentifier.BACKDOOR_EXHAUSTIVE and num_iterations > max_iterations:
+                    self.logger.warning(f"Max number of iterations {max_iterations} reached.")
                     break
-        else:
-            raise ValueError(f"Identifier method {self.method_name} not supported. Try one of the following: {CausalIdentifier.VALID_METHOD_NAMES}")
-        #causes_t = self._graph.get_causes(self.treatment_name)
-        #causes_y = self._graph.get_causes(self.outcome_name, remove_edges={'sources':self.treatment_name, 'targets':self.outcome_name})
-        #common_causes = list(causes_t.intersection(causes_y))
-        #self.logger.info("Common causes of treatment and outcome:" + str(common_causes))
-        observed_backdoor_sets = [ bset for bset in backdoor_sets if self._graph.all_observed(bset["backdoor_set"])]
-        if len(observed_backdoor_sets)==0:
-            return backdoor_sets
-        else:
-            return observed_backdoor_sets
+            # If the backdoor method is `maximal-adjustment` or `minimal-adjustment`, return the first found adjustment set.
+            if method_name in {CausalIdentifier.BACKDOOR_DEFAULT, CausalIdentifier.BACKDOOR_MAX, CausalIdentifier.BACKDOOR_MIN} and found_valid_adjustment_set:
+                break
+            # If all variables are observed, and the biggest eligible set
+            # does not satisfy backdoor, then none of its subsets will.
+            if method_name in {CausalIdentifier.BACKDOOR_DEFAULT, CausalIdentifier.BACKDOOR_MAX} and all_nodes_observed:
+                break
+            if num_iterations > max_iterations:
+                self.logger.warning(f"Max number of iterations {max_iterations} reached. Could not find a valid backdoor set.")
+                break
+        return backdoor_sets, found_valid_adjustment_set
+
 
     def get_default_backdoor_set_id(self, backdoor_sets_dict):
         # Adding a None estimand if no backdoor set found
         if len(backdoor_sets_dict) == 0:
             return None
-        max_set_length = -1
+
+        # Default set contains minimum possible number of instrumental variables, to prevent lowering variance in the treatment variable.
+        instrument_names = set(self._graph.get_instruments(self.treatment_name, self.outcome_name))
+        iv_count_dict = {key: len(set(bdoor_set).intersection(instrument_names)) for key, bdoor_set in backdoor_sets_dict.items()}
+        min_iv_count = min(iv_count_dict.values())
+        min_iv_keys = {key for key, iv_count in iv_count_dict.items() if iv_count == min_iv_count}
+        min_iv_backdoor_sets_dict = {key: backdoor_sets_dict[key] for key in min_iv_keys}
+
+        # Default set is the one with the least number of adjustment variables (optimizing for efficiency)
+        min_set_length = 1000000
         default_key = None
-        # Default set is the one with the most number of adjustment variables (optimizing for minimum (unknown) bias not for efficiency)
-        for key, bdoor_set in backdoor_sets_dict.items():
-            if len(bdoor_set) > max_set_length:
-                max_set_length = len(bdoor_set)
+        for key, bdoor_set in min_iv_backdoor_sets_dict.items():
+            if len(bdoor_set) < min_set_length:
+                min_set_length = len(bdoor_set)
                 default_key = key
         return default_key
 
@@ -304,32 +381,15 @@ class CausalIdentifier:
             proceed_when_unidentifiable = self._proceed_when_unidentifiable
         is_identified = [ self._graph.all_observed(bset["backdoor_set"]) for bset in backdoor_sets ]
 
-        if all(is_identified):
-            self.logger.info("All common causes are observed. Causal effect can be identified.")
+        if any(is_identified):
+            self.logger.info("Causal effect can be identified.")
             backdoor_sets_arr = [list(
                 bset["backdoor_set"])
-                for bset in backdoor_sets]
+                for bset in backdoor_sets
+                if self._graph.all_observed(bset["backdoor_set"]) ]
         else: # there is unobserved confounding
-            self.logger.warning("If this is observed data (not from a randomized experiment), there might always be missing confounders. Causal effect cannot be identified perfectly.")
-            response = False # user response
-            if proceed_when_unidentifiable:
-                self.logger.info(
-                    "Continuing by ignoring these unobserved confounders because proceed_when_unidentifiable flag is True."
-                )
-            else:
-                response= cli.query_yes_no(
-                    "WARN: Do you want to continue by ignoring any unobserved confounders? (use proceed_when_unidentifiable=True to disable this prompt)",
-                    default=None
-                )
-                if response is False:
-                    self.logger.warn("Identification failed due to unobserved variables.")
-                    backdoor_sets_arr = []
-            if proceed_when_unidentifiable or response is True:
-                max_paths_blocked = max( bset['num_paths_blocked_by_observed_nodes'] for bset in backdoor_sets)
-                backdoor_sets_arr = [list(
-                    self._graph.filter_unobserved_variables(bset["backdoor_set"]))
-                    for bset in backdoor_sets
-                    if bset["num_paths_blocked_by_observed_nodes"]==max_paths_blocked]
+            self.logger.warning("Backdoor identification failed.")
+            backdoor_sets_arr = []
 
         for i in range(len(backdoor_sets_arr)):
             backdoor_estimand_expr = self.construct_backdoor_estimand(
@@ -340,20 +400,59 @@ class CausalIdentifier:
             backdoor_variables_dict["backdoor"+str(i+1)] = backdoor_sets_arr[i]
         return estimands_dict, backdoor_variables_dict
 
-    def identify_frontdoor(self):
+    def identify_frontdoor(self, dseparation_algo="default"):
         """ Find a valid frontdoor variable if it exists.
 
         Currently only supports a single variable frontdoor set.
         """
         frontdoor_var = None
-        frontdoor_paths = self._graph.get_all_directed_paths(self.treatment_name, self.outcome_name)
+        frontdoor_paths = None
+        fdoor_graph = None
+        if dseparation_algo == "default":
+            cond1_graph = self._graph.do_surgery(self.treatment_name,
+                    remove_incoming_edges=True)
+            bdoor_graph1 = self._graph.do_surgery(self.treatment_name,
+                    remove_outgoing_edges=True)
+        elif dseparation_algo == "naive":
+            frontdoor_paths = self._graph.get_all_directed_paths(self.treatment_name, self.outcome_name)
+        else:
+            raise ValueError(f"d-separation algorithm {dseparation_algo} is not supported")
+
+
         eligible_variables = self._graph.get_descendants(self.treatment_name) \
-            - set(self.outcome_name)
+            - set(self.outcome_name) \
+            - set(self._graph.get_descendants(self.outcome_name))
         # For simplicity, assuming a one-variable frontdoor set
         for candidate_var in eligible_variables:
-            is_valid_frontdoor = self._graph.check_valid_frontdoor_set(self.treatment_name,
-                    self.outcome_name, parse_state(candidate_var), frontdoor_paths=frontdoor_paths)
-            self.logger.debug("Candidate frontdoor set: {0}, is_dseparated: {1}".format(candidate_var, is_valid_frontdoor))
+            # Cond 1: All directed paths intercepted by candidate_var
+            cond1 = self._graph.check_valid_frontdoor_set(
+                self.treatment_name, self.outcome_name,
+                parse_state(candidate_var),
+                frontdoor_paths=frontdoor_paths,
+                new_graph=cond1_graph,
+                dseparation_algo=dseparation_algo)
+            self.logger.debug("Candidate frontdoor set: {0}, is_dseparated: {1}".format(candidate_var, cond1))
+            if not cond1:
+                continue
+            # Cond 2: No confounding between treatment and candidate var
+            cond2 = self._graph.check_valid_backdoor_set(
+                self.treatment_name, parse_state(candidate_var),
+                set(),
+                backdoor_paths=None,
+                new_graph= bdoor_graph1,
+                dseparation_algo=dseparation_algo)
+            if not cond2:
+                continue
+            # Cond 3: treatment blocks all confounding between candidate_var and outcome
+            bdoor_graph2 = self._graph.do_surgery(candidate_var,
+                    remove_outgoing_edges=True)
+            cond3 = self._graph.check_valid_backdoor_set(
+                parse_state(candidate_var), self.outcome_name,
+                self.treatment_name,
+                backdoor_paths=None,
+                new_graph= bdoor_graph2,
+                dseparation_algo=dseparation_algo)
+            is_valid_frontdoor = cond1 and cond2 and cond3
             if is_valid_frontdoor:
                 frontdoor_var = candidate_var
                 break
@@ -413,19 +512,6 @@ class CausalIdentifier:
         backdoor_variables_dict["backdoor"] = backdoor_variables_dict.get(str(default_backdoor_id), None)
         return backdoor_variables_dict
 
-    def get_default_backdoor_set_id(self, backdoor_sets_dict):
-        # Adding a None estimand if no backdoor set found
-        if len(backdoor_sets_dict) == 0:
-            return None
-        max_set_length = -1
-        default_key = None
-        # Default set is the one with the most number of adjustment variables (optimizing for minimum (unknown) bias not for efficiency)
-        for key, bdoor_set in backdoor_sets_dict.items():
-            if len(bdoor_set) > max_set_length:
-                max_set_length = len(bdoor_set)
-                default_key = key
-        return default_key
-
     def construct_backdoor_estimand(self, estimand_type, treatment_name,
                                     outcome_name, common_causes):
         # TODO: outputs string for now, but ideally should do symbolic
@@ -586,7 +672,8 @@ class IdentifiedEstimand:
                  mediator_variables=None,
                  mediation_first_stage_confounders=None,
                  mediation_second_stage_confounders=None,
-                 default_backdoor_id=None, identifier_method=None):
+                 default_backdoor_id=None, identifier_method=None,
+                 no_directed_path=False):
         self.identifier = identifier
         self.treatment_variable = parse_state(treatment_variable)
         self.outcome_variable = parse_state(outcome_variable)
@@ -600,6 +687,7 @@ class IdentifiedEstimand:
         self.estimands = estimands
         self.default_backdoor_id = default_backdoor_id
         self.identifier_method = identifier_method
+        self.no_directed_path = no_directed_path
 
     def set_identifier_method(self, identifier_name):
         self.identifier_method = identifier_name
@@ -614,7 +702,7 @@ class IdentifiedEstimand:
         if key is None:
             if self.identifier_method and self.identifier_method.startswith("backdoor"):
                 return self.backdoor_variables[self.identifier_method]
-            elif self.backdoor_variables is not None:
+            elif self.backdoor_variables is not None and len(self.backdoor_variables) > 0:
                 return self.backdoor_variables[self.default_backdoor_id]
             else:
                 return []
@@ -656,6 +744,12 @@ class IdentifiedEstimand:
             )
 
     def __str__(self, only_target_estimand=False, show_all_backdoor_sets=False):
+        if self.no_directed_path:
+            s = "No directed path from {0} to {1} in the causal graph.".format(
+                    self.treatment_variable,
+                    self.outcome_variable)
+            s += "\nCausal effect is zero."
+            return s
         s = "Estimand type: {0}\n".format(self.estimand_type)
         i = 1
         has_valid_backdoor = sum("backdoor" in key for key in self.estimands.keys())
@@ -676,7 +770,7 @@ class IdentifiedEstimand:
                 s += " (Default)"
             s += "\n"
             if v is None:
-                s += "No such variable found!\n"
+                s += "No such variable(s) found!\n"
             else:
                 sp_expr_str = sp.pretty(v["estimand"], use_unicode=True)
                 s += "Estimand expression:\n{0}\n".format(sp_expr_str)
