@@ -56,6 +56,7 @@ def linear_dataset(beta, num_common_causes, num_samples, num_instruments=0,
                    num_discrete_common_causes=0,
                    num_discrete_instruments=0,
                    num_discrete_effect_modifiers=0,
+                   stddev_treatment_noise = 1,
                    one_hot_encode = False):
     W, X, Z, FD, c1, c2, ce, cz, cfd1, cfd2 = [None]*10
     W_with_dummy, X_with_categorical  = (None, None)
@@ -67,8 +68,8 @@ def linear_dataset(beta, num_common_causes, num_samples, num_instruments=0,
     num_cont_instruments = num_instruments -num_discrete_instruments
     num_cont_effect_modifiers = num_effect_modifiers - num_discrete_effect_modifiers
     if num_common_causes > 0:
-        range_c1 = max(beta)*0.5
-        range_c2 = max(beta)*0.5
+        range_c1 = 0.5 + max(abs(beta))*0.5
+        range_c2 = 0.5 + max(abs(beta))*0.5
         means = np.random.uniform(-1, 1, num_common_causes)
         cov_mat = np.diag(np.ones(num_common_causes))
         W = np.random.multivariate_normal(means, cov_mat, num_samples)
@@ -78,7 +79,7 @@ def linear_dataset(beta, num_common_causes, num_samples, num_instruments=0,
         c2 = np.random.uniform(0, range_c2, W_with_dummy.shape[1])
 
     if num_instruments > 0:
-        range_cz = beta
+        range_cz = 1 + max(abs(beta))
         p = np.random.uniform(0, 1, num_instruments)
         Z = np.zeros((num_samples, num_instruments))
         for i in range(num_instruments):
@@ -90,7 +91,7 @@ def linear_dataset(beta, num_common_causes, num_samples, num_instruments=0,
         cz = np.random.uniform(range_cz - (range_cz * 0.05),
                 range_cz + (range_cz * 0.05), (num_instruments, num_treatments))
     if num_effect_modifiers >0:
-        range_ce = beta*0.5
+        range_ce = 0.5 + max(abs(beta))*0.5
         means = np.random.uniform(-1, 1, num_effect_modifiers)
         cov_mat = np.diag(np.ones(num_effect_modifiers))
         X = np.random.multivariate_normal(means, cov_mat, num_samples)
@@ -100,7 +101,7 @@ def linear_dataset(beta, num_common_causes, num_samples, num_instruments=0,
         ce = np.random.uniform(0, range_ce, X_with_categorical.shape[1])
     # TODO - test all our methods with random noise added to covariates (instead of the stochastic treatment assignment)
 
-    t = np.random.normal(0, 1, (num_samples, num_treatments))
+    t = np.random.normal(0, stddev_treatment_noise, (num_samples, num_treatments))
     if num_common_causes > 0:
         t += W_with_dummy @ c1  # + np.random.normal(0, 0.01)
     if num_instruments > 0:
@@ -111,8 +112,8 @@ def linear_dataset(beta, num_common_causes, num_samples, num_instruments=0,
 
     # Generating frontdoor variables if asked for
     if num_frontdoor_variables > 0:
-        range_cfd1 = max(beta)*0.5
-        range_cfd2 = max(beta)*0.5
+        range_cfd1 = max(abs(beta))*0.5
+        range_cfd2 = max(abs(beta))*0.5
         cfd1 = np.random.uniform(0, range_cfd1, (num_treatments, num_frontdoor_variables))
         cfd2 = np.random.uniform(0, range_cfd2, num_frontdoor_variables)
         FD_noise = np.random.normal(0, 1, (num_samples, num_frontdoor_variables))
@@ -220,7 +221,7 @@ def simple_iv_dataset(beta, num_samples,
 
     c1 = np.random.uniform(0,1, (num_common_causes, num_treatments))
     c2 = np.random.uniform(0,1, num_common_causes)
-    range_cz = beta # cz is much higher than c1 and c2
+    range_cz = 1 + max(abs(beta)) # cz is much higher than c1 and c2
     cz = np.random.uniform(range_cz - (range_cz * 0.05),
                 range_cz + (range_cz * 0.05), (num_instruments, num_treatments))
     W = np.random.uniform(0, 1, (num_samples, num_common_causes))
@@ -270,14 +271,10 @@ def simple_iv_dataset(beta, num_samples,
 
 def create_dot_graph(treatments, outcome, common_causes,
         instruments, effect_modifiers=[], frontdoor_variables=[]):
-    dot_graph = ('digraph {{'
-                 ' U[label="Unobserved Confounders"];'
-                 ' U->{0};'
-                 ).format(outcome)
+    dot_graph = 'digraph {'
     for currt in treatments:
         if len(frontdoor_variables) == 0:
             dot_graph += '{0}->{1};'.format(currt, outcome)
-        dot_graph +=  'U->{0};'.format(currt)
         dot_graph +=  " ".join([v + "-> " + currt + ";" for v in common_causes])
         dot_graph += " ".join([v + "-> " + currt + ";" for v in instruments])
         dot_graph += " ".join([currt + "-> " + v + ";" for v in frontdoor_variables])
@@ -295,17 +292,14 @@ def create_gml_graph(treatments, outcome, common_causes,
         instruments, effect_modifiers=[], frontdoor_variables=[]):
     gml_graph = ('graph[directed 1'
                  'node[ id "{0}" label "{0}"]'
-                 'node[ id "{1}" label "{1}"]'
-                 'edge[source "{1}" target "{0}"]'
-                 ).format(outcome, "Unobserved Confounders")
+                 ).format(outcome)
 
     gml_graph +=  " ".join(['node[ id "{0}" label "{0}"]'.format(v) for v in common_causes])
     gml_graph += " ".join(['node[ id "{0}" label "{0}"]'.format(v) for v in instruments])
     gml_graph += " ".join(['node[ id "{0}" label "{0}"]'.format(v) for v in frontdoor_variables])
     for currt in treatments:
         gml_graph += ('node[ id "{0}" label "{0}"]'
-                     'edge[source "{1}" target "{0}"]'
-                     ).format(currt, "Unobserved Confounders")
+                     ).format(currt)
         if len(frontdoor_variables) == 0:
             gml_graph += 'edge[source "{0}" target "{1}"]'.format(currt, outcome)
         gml_graph +=  " ".join(['edge[ source "{0}" target "{1}"]'.format(v, currt) for v in common_causes])
