@@ -64,13 +64,13 @@ class ConfounderDistributionInterpreter(VisualInterpreter):
         """
 
         cols = self.estimator._observed_common_causes_names + self.estimator._treatment_name
-        df = self.estimator._data[cols]
+        df = self.estimator._data[cols].copy()
         treated = self.estimator._treatment_name[0]
         propensity = self.estimate.propensity_scores
 
         # add weight column
-        df["weight"] = df[treated] * (propensity) ** (-1) + (1 - df[treated]) * (1 - propensity) ** (-1)
-
+        df.loc[:,"weight"] = df.loc[:, treated] * (propensity) ** (-1) + (1 - df.loc[:, treated]) * (1 - propensity) ** (-1)
+        
         # before weights are applied we count number rows in each category
         # which is equivalent to summing over weight=1
         barplot_df_before = df.groupby([self.var_name, treated]).size().reset_index(name="count")
@@ -86,13 +86,13 @@ class ConfounderDistributionInterpreter(VisualInterpreter):
         fig, (ax1, ax2) = plt.subplots(1, 2, figsize=self.fig_size)
         iterable = zip([barplot_df_before, barplot_df_after], [ax1, ax2], [title1, title2])
         for plot_df, ax, title in iterable:
-            aggreagted_not_treated = plot_df[plot_df["treat"] == False]
-            aggreagted_treated = plot_df[plot_df["treat"] == True]
+            aggregated_not_treated = plot_df[plot_df[treated] == False].reset_index()
+            aggregated_treated = plot_df[plot_df[treated] == True].reset_index()
 
-            labels = aggreagted_not_treated[self.var_name]
-            not_treated_counts = aggreagted_not_treated['count']
+            labels = aggregated_not_treated[self.var_name].astype('float')
+            not_treated_counts = aggregated_not_treated['count']
 
-            treated_counts = aggreagted_treated['count']
+            treated_counts = aggregated_treated['count']
             self.discrete_dist_plot(labels, not_treated_counts, treated_counts, ax, title,
                                     self.var_name, self.font_size)
 
